@@ -34,7 +34,7 @@ class Xontrib(tp.NamedTuple):
 
     module: str
     """path to the xontrib module"""
-    distribution: "tp.Optional[Distribution]" = None
+    distribution: "Distribution | None" = None
     """short description about the xontrib."""
 
     def get_description(self):
@@ -269,6 +269,8 @@ def xontribs_unload(
     ----------
     names
         name of xontribs to unload
+    verbose : -v, --verbose
+        verbose output
 
     Notes
     -----
@@ -303,6 +305,8 @@ def xontribs_reload(
     ----------
     names
         name of xontribs to reload
+    verbose : -v, --verbose
+        verbose output
     """
     for name in names:
         if verbose:
@@ -330,7 +334,7 @@ def xontribs_loaded():
     return [k for k, xontrib in get_xontribs().items() if xontrib.is_loaded]
 
 
-def xontribs_list(to_json=False):
+def xontribs_list(to_json=False, _stdout=None):
     """List installed xontribs and show whether they are loaded or not
 
     Parameters
@@ -341,7 +345,7 @@ def xontribs_list(to_json=False):
     data = xontrib_data()
     if to_json:
         s = json.dumps(data)
-        print(s)
+        return s
     else:
         nname = max([6] + [len(x) for x in data])
         s = ""
@@ -356,7 +360,7 @@ def xontribs_list(to_json=False):
             else:
                 s += "{RED}not-loaded{RESET}"
             s += "\n"
-        print_color(s[:-1])
+        print_color(s[:-1], file=_stdout)
 
 
 def _get_xontrib_entrypoints() -> "tp.Iterable[EntryPoint]":
@@ -365,7 +369,11 @@ def _get_xontrib_entrypoints() -> "tp.Iterable[EntryPoint]":
     name = "xonsh.xontribs"
     entries = metadata.entry_points()
     # for some reason, on CI (win py3.8) atleast, returns dict
-    group = entries.select(group=name) if hasattr(entries, "select") else entries.get(name, [])  # type: ignore
+    group = (
+        entries.select(group=name)
+        if hasattr(entries, "select")
+        else entries.get(name, [])  # type: ignore
+    )
     yield from group
 
 
@@ -395,7 +403,7 @@ class XontribAlias(ArgParserAlias):
         parser.add_command(xontribs_load, prog="load")
         parser.add_command(xontribs_unload, prog="unload")
         parser.add_command(xontribs_reload, prog="reload")
-        parser.add_command(xontribs_list, prog="list")
+        parser.add_command(xontribs_list, prog="list", default=True)
         return parser
 
 
